@@ -1,7 +1,7 @@
 # coding=utf-8
 from __future__ import absolute_import, print_function
 
-import time
+import gevent
 import traceback
 
 import redis
@@ -158,17 +158,21 @@ class MQ(object):
             if success != 'true':
                 return
 
-            for target, data4in in self._sendP2PMessages(data):
-                target_queue = f'{queue}-{target}'
+            for target, data4in in self._sendP2PMessages(queue, data):
                 logger.debug(f'send p2p message to {target}: {data4in}')
                 self.client.xadd(
-                    target_queue, data4in, id=messageID, maxlen=maxlen, approximate=(not trimImmediately)
+                    target, data4in, id=messageID, maxlen=maxlen, approximate=(not trimImmediately)
                 )
         else:
             return self.client.xadd(
                 queue, data, id=messageID, maxlen=maxlen, approximate=(not trimImmediately)
             )
 
+    def sendEvent(self, queue, data, messageID="*", maxlen=1000, trimImmediately=False):
+        return self.client.xadd(
+            queue, data, id=messageID, maxlen=maxlen, approximate=(not trimImmediately)
+        )
+
     def _getP2PConnections(self):
         from suanpan.graph import graph
 
@@ -177,19 +181,36 @@ class MQ(object):
         for port in node.metadata['def'].ports:
             if port.ioType == 'out':
                 port_id = port.uuid
+                to_dashboard = False
                 connections = graph.getOutputConnections(g.nodeId, port_id)
-                self._outputConnections[port_id] = connections
+                for tgt in connections:
+                    node = graph.getNodeById(tgt.process)
+                    if node.metadata['def'].type in [22, 10002]:
+                        to_dashboard = True
+                        break
 
-    def _sendP2PMessages(self, data):
+                self._outputConnections[port_id] = (to_dashboard, connections)
+
+    def _sendP2PMessages(self, queue, data):
+        send_master = False
         for port_id, connections in self._outputConnections.items():
-            for tgt in connections:
-                d = data.get(port_id)
-                if d:
-                    data4in = {"id": data['request_id'], tgt.port: d}
-                    extra = data.get('extra')
-                    if extra:
-                        data4in["extra"] = extra
-                    yield tgt.process, data4in
+            to_dashboard, connections = connections
+            send_master = send_master or to_dashboard
+            if not to_dashboard:
+                for tgt in connections:
+                    d = data.get(port_id)
+                    if d:
+                        data4in = {"id": data['request_id'], tgt.port: d}
+                        extra = data.get('extra')
+                        if extra:
+                            data4in["extra"] = extra
+                        target_queue = f'{queue}-{tgt.process}'
+                        yield target_queue, data4in
+                        data.pop(port_id, None)
+
+        if send_master:
+            # connection to dashboard, data must send to master
+            yield queue, data
 
     def recvMessages(
             self,
@@ -251,19 +272,64 @@ class MQ(object):
 
                 errCallback(e)
                 logger.debug(f"Error in receiving messages. Wait {errDelay}s")
-                time.sleep(errDelay)
+                gevent.sleep(errDelay)
                 self.createQueue(queue, group=group, existOk=True)
                 continue
 
             if not messages:
                 logger.debug(f"Received no messages. Wait {delay}s")
-                time.sleep(delay)
+                gevent.sleep(delay)
                 continue
 
             for message in messages:
                 yield message
                 self.client.xack(queue, group, message["id"])
 
+    def recvEvents(
+            self,
+            queue,
+            block=False,
+            count=1,
+            consumeID="$",
+    ):
+        block = None if not block else 0 if block is True else block
+        events = self.client.xread({queue: consumeID}, count=count, block=block)
+        events = list(self._parseMessagesGenerator(events, None))
+
+        return [event for event in events if event["data"]]
+
+    def subscribeEvent(
+            self,
+            queue,
+            block=True,
+            count=1,
+            consumeID="$",
+            delay=0,
+            errDelay=1,
+            errCallback=logger.error,
+    ):
+        logger.debug("Subscribing Event")
+        while True:
+            try:
+                events = self.recvEvents(queue, block=block, count=count, consumeID=consumeID)
+            except Exception as e:  # pylint: disable=broad-except
+                if self._stop_event.is_set():
+                    logger.debug(f"stop event is set, break.")
+                    break
+
+                errCallback(e)
+                logger.debug(f"Error in receiving events. Wait {errDelay}s")
+                gevent.sleep(errDelay)
+                continue
+
+            if not events:
+                # logger.debug(f"Received no events. Wait {delay}s")
+                gevent.sleep(delay)
+                continue
+
+            for event in events:
+                yield event
+
     def recvPendingMessagesInfo(
             self, queue, group="default", consumer="unknown", start="-", end="+", count=None
     ):
