@@ -642,7 +642,6 @@ class StreamPubsubLoop(StreamBaseLoop):
             self.handle(_event)
 
     def handle(self, event):
-        logger.debug(f"get event: {event}")
         topic = event.get('event_name')
         if not topic:
             return
@@ -654,6 +653,7 @@ class StreamPubsubLoop(StreamBaseLoop):
         try:
             handler = self.stream.eventHooks.get(topic)
             if handler:
+                logger.debug(f"handle event: {event}")
                 d = json.loads(data)
                 handler(d)
         except Exception:
